#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for scicat ingestor """

import os
import socket
import tango
import json
import time
import re

# from sardana.macroserver.macro import Macro


def append_scicat_dataset(macro, status_info=True, reingest=False):
    """ append scan name to the dataset scan list file

    :param macro: hook macro
    :type macro: :class:`sardana.macroserver.macro.Macro`
    :param status_info: status info flag
    :type status_info: :obj:`bool`
    :param reingest: reingest flag
    :type reingest: :obj:`bool`
    :return: scan name if appended
    :rtype: :obj:`str`
    """
    sname = ""
    append = get_env_var(macro, "AppendSciCatDataset", None)
    nxsappend = get_env_var(macro, "NXSAppendSciCatDataset", None)
    if not append and not (nxsappend and reingest):
        return sname

    sfl = macro.getEnv('ScanFile')
    sid = macro.getEnv('ScanID')

    # find scan name for the master file
    if isinstance(sfl, str):
        sfl = [sfl]
    scanname = ""
    nexus = False
    if sfl and isinstance(sfl, list) or isinstance(sfl, tuple):
        for sf in sfl:
            scname, ext = os.path.splitext(str(sf))
            if ext in [".nxs", ".nx", ".h5", ".ndf"] and scname:
                scanname = str(scname)
                nexus = True
                break
        if not scanname:
            for sf in sfl:
                scname, ext = os.path.splitext(str(sf))
                if ext in [".fio"] and scname:
                    scanname = str(scname)
                    nxsappend = None
                    break
        if not scanname:
            for sf in sfl:
                scname, ext = os.path.splitext(str(sf))
                if scname:
                    scanname = str(scname)
                    nxsappend = None
                    break
    if scanname and (not nxsappend or reingest):
        appendentry = False
        entryname = "scan"
        try:
            nsd = get_env_var(macro, "NeXusSelectorDevice", None)
            if nsd:
                tnsd = tango.DeviceProxy(nsd)
                appendentry = bool(tnsd.appendentry)
                variables = json.loads(tnsd.configvariables)
                if isinstance(variables, dict) and "entryname" in variables:
                    entryname = variables["entryname"]
        except Exception:
            macro.warning("NeXusSelectorDevice is not available")
        scanname2 = scanname
        if nexus and '{ScanID' in scanname:
            try:
                sname = scanname.format(ScanID=sid)
                scanname2 = os.path.commonprefix(
                    [scanname.format(ScanID=11111111),
                     scanname.format(ScanID=99999999)])
                if scanname2.endswith("_"):
                    scanname2 = scanname2[:-1]
            except Exception:
                sname = "%s_%05i" % (scanname, sid)
        elif nexus and '%' in scanname:
            try:
                sname = scanname % sid
                try:
                    scanname2 = os.path.commonprefix(
                        [scanname % 11111111,
                         scanname % 99999999])
                    if scanname2.endswith("_"):
                        scanname2 = scanname2[:-1]
                except Exception:
                    pass
            except Exception:
                sname = "%s_%05i" % (scanname, sid)
        elif not nexus or appendentry is False:
            sname = "%s_%05i" % (scanname, sid)
        else:
            sname = "%s::/%s_%05i;%s_%05i" % (
                scanname, entryname, sid, scanname, sid)
        if reingest:
            sname = "%s:%s" % (sname, time.time())
        else:
            skip_acq = get_env_var(macro, 'NeXusSkipAcquisitionModes', [])
            if isinstance(skip_acq, str):
                skip_acq = re.split(r"[-;,.\s]\s*", skip_acq)
            if "INIT" in skip_acq:
                sname = "%s:%s" % (sname, time.time())

        fdir = macro.getEnv('ScanDir')
        fdir = os.path.abspath(fdir)
        snmode = get_env_var(macro, 'ScanNames', None)
        nometa = get_env_var(macro, 'ScanNamesNoMetadata', False)
        nogrouping = get_env_var(macro, 'ScanNamesNoGrouping', False)
        pdir = None
        if snmode is not None:
            if bool(snmode):
                fdir = os.path.dirname(os.path.abspath(fdir))
            elif appendentry is False:
                fdir, pdir = os.path.split(os.path.abspath(fdir))
        if pdir:
            sname = "%s/%s" % (pdir, sname)

        # auto grouping
        grouping = bool(get_env_var(macro, 'SciCatAutoGrouping', False))
        if grouping or pdir:
            commands = []
            try:
                sm = dict(get_env_var(macro, 'SciCatMeasurements', {}))
                if not isinstance(sm, dict):
                    sm = {}
            except Exception:
                sm = {}

            if fdir in sm.keys():
                cgrp = sm[fdir]
                if cgrp != scanname2:
                    if not nogrouping and not nometa:
                        commands.append("__command__ stop")
                        commands.append("%s:%s" % (cgrp, time.time()))
                        commands.append("__command__ start %s" % scanname2)
            else:
                if not nogrouping and not nometa:
                    commands.append("__command__ start %s" % scanname2)
            if not nometa:
                commands.append(sname)
            if not nogrouping and not nometa:
                commands.append("__command__ stop")
            if not nogrouping:
                commands.append("%s:%s" % (scanname2, time.time()))
            sname = "\n".join(commands)

            if not nogrouping and not nometa:
                if fdir in sm.keys():
                    sm.pop(fdir)
            macro.setEnv('SciCatMeasurements', sm)
        if sname:
            append_scicat_record(macro, sname, status_info=True)
    return sname


def append_scicat_record(macro, sname, status_info=True):
    """ append scan name to the dataset scan list file

    :param macro: hook macro
    :type macro: :class:`sardana.macroserver.macro.Macro`
    :param sname: scingestor record
    :type sname: :obj:`bool`
    :param macro: status info flag
    :type macro: :obj:`bool`
    :return: scan name if appended
    :rtype: :obj:`str`
    """
    # get beamtime id
    fdir = macro.getEnv('ScanDir')
    fdir = os.path.abspath(fdir)
    if get_env_var(macro, 'ScanNames', None) is not None:
        fdir = os.path.dirname(os.path.abspath(fdir))
    bmtfpath = get_env_var(macro, "BeamtimeFilePath", "/gpfs/current")
    bmtfprefix = get_env_var(
        macro, "BeamtimeFilePrefix", "beamtime-metadata-")
    bmtfext = get_env_var(macro, "BeamtimeFileExt", ".json")
    beamtimeid = beamtime_id(fdir, bmtfpath, bmtfprefix, bmtfext)
    beamtimeid = beamtimeid or "00000000"

    # get scicat dataset list file name
    defprefix = "scicat-datasets-"
    defaulthost = get_env_var(macro, "SciCatDatasetListFileLocal", None)
    hostname = None
    if defaulthost:
        hostname = socket.gethostname()
    if hostname and hostname is not True and hostname.lower() != "true":
        defprefix = "%s%s-" % (defprefix, str(hostname))
    dslprefix = get_env_var(
        macro, "SciCatDatasetListFilePrefix", defprefix)
    dslext = get_env_var(macro, "SciCatDatasetListFileExt", ".lst")
    dslfile = "%s%s%s" % (dslprefix, beamtimeid, dslext)
    if fdir:
        dslfile = os.path.join(fdir, dslfile)

    # append the scan name to the list file
    with open(dslfile, "a+") as fl:
        fl.write("\n%s" % sname)
    if status_info:
        macro.output("Appending '" + sname + "' to " + dslfile)


def beamtime_id(fpath, bmtfpath, bmtfprefix, bmtfext):
    """ code for beamtimeid  datasource

    :param fpath:  scan file directory
    :type fpath: :obj:`str`
    :param bmtfpath:  beamtime file directory
    :type bmtfpath: :obj:`str`
    :param bmtfprefix:  beamtime file prefix
    :type bmtfprefix: :obj:`str`
    :param bmtfext:  beamtime file postfix
    :type bmtfext: :obj:`str`
    :returns: beamtime id
    :rtype: :obj:`str`
    """
    result = ""
    if fpath.startswith(bmtfpath):
        try:
            if os.path.isdir(bmtfpath):
                btml = [fl for fl in os.listdir(bmtfpath)
                        if (fl.startswith(bmtfprefix)
                            and fl.endswith(bmtfext))]
                result = btml[0][len(bmtfprefix):-len(bmtfext)]
        except Exception:
            pass
    return result


def get_env_var(macro, name, defvalue):
    """ get environment variable

    :param macro: hook macro
    :type macro: :class:`sardana.macroserver.macro.Macro`
    :param name: variable name
    :type name: :obj:`str`
    :param defvalue: default value
    :type defvalue: :obj:`str`
    """
    try:
        return macro.getEnv(name)
    except Exception:
        return defvalue



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for lambdavds """

import json

try:
    from . import common
except Exception:
    import common


def nm_triggermode_cb(commonblock, name, triggermode,
                      translations, saveallimages,
                      filepostfix, framenumbers,
                      height, width, opmode,
                      savefilename, filename, entryname,
                      insname="instrument",
                      shortdetpath=None):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: component name
    :type name: :obj:`str`
    :param triggermode:  trigger mode
    :type triggermode: :obj:`int` or :obj:`str`
    :param translations: json dictionary with translations
    :type translations: :obj:`str`
    :param saveallimages: save all images flag
    :type saveallimages: :obj:`int` or :obj:`bool`
    :param filepostfix: filename postfix
    :type filepostfix:  :obj:`str`
    :param framenumbers: a number of frames
    :type framenumbers: :obj:`int`
    :param height: height of the image
    :type height: :obj:`int`
    :param width: width of the image
    :type width: :obj:`int`
    :param opmode: operation mode,
                   i.e. 1="int8", 6="int8", 12="int16", 24="int32"
    :type opmode:  :obj:`int`
    :param savefilename: savefilename
    :type savefilename: :obj:`str`
    :param filename: master file name
    :type filename: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param shortdetpath: shortdetpath
    :type shortdetpath: :obj:`bool`
    :returns:  triggermode
    :rtype: :obj:`str` or :obj:`int`
    """
    if saveallimages:
        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
        dtm = {1: "int8", 6: "int8", 12: "int16", 24: "int32"}
        try:
            dtype = dtm[opmode]
        except Exception:
            dtype = "int32"

        modoffsets = json.loads(translations)
        totalheight = 0
        totalwidth = 0
        totalframenumbers = 0
        modsize = len(list(modoffsets.keys()))
        for offset in modoffsets.values():
            totalframenumbers = max(
                totalframenumbers, framenumbers + offset[0])
            totalheight = max(totalheight, height + offset[1])
            totalwidth = max(totalwidth, width + offset[2])
        unlimited = False
        if totalframenumbers == framenumbers:
            unlimited = True

        path = ""
        if filename:
            sfname = (filename).split("/")
            path = sfname[-1].split(".")[0] + "/"
            if shortdetpath is None and \
                    len(sfname) > 1 and sfname[-2] == path[:-1]:
                path = ""
            elif shortdetpath:
                path = ""

        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
            if root.h5object.__class__.__name__ == "File":
                import nxstools.h5pywriter as nxw
            else:
                import nxstools.h5cppwriter as nxw
        else:
            raise Exception("Writer cannot be found")

        en = root.open(entryname)
        ins = en.open(insname)
        det = ins.open(name)
        npath = "/entry/instrument/detector/data"
        vfl = nxw.virtual_field_layout(
            [totalframenumbers, totalheight, totalwidth], dtype)
        for modulename, offset in modoffsets.items():
            mfilename = path + name + "/" + str(savefilename)
            if modsize != 1:
                mfilename += "_" + modulename
            mfilename += "." + str(filepostfix)
            ef = nxw.target_field_view(
                mfilename, npath, [framenumbers, height, width], dtype)
            if unlimited:
                vfl.add(
                    (slice(None, nxw.unlimited()),
                     slice(offset[1], height + offset[1]),
                     slice(offset[2], width + offset[2])),
                    ef,
                    (slice(None, nxw.unlimited()),
                     slice(None), slice(None)))
            else:
                vfl.add(
                    (slice(offset[0], framenumbers + offset[0]),
                     slice(offset[1], height + offset[1]),
                     slice(offset[2], width + offset[2])),
                    ef,
                    (slice(None), slice(None), slice(None)))
        det.create_virtual_field("data", vfl)
    return triggermode


def savefilename_cb(commonblock, savefilename, savefilename_str):
    """ code for savefilename_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param savefilename:  name of saved file
    :type savefilename: :obj:`str`
    :param savefilename_str: name of savefilename datasource
    :type savefilename_str: :obj:`str`
    :returns:   name of saved file
    :rtype: :obj:`str`
    """
    return common.blockitem_add(
        commonblock, savefilename_str, savefilename)


def framenumbers_cb(commonblock, framenumbers, framenumbers_str):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param framenumbers:  number of frames
    :type framenumbers: :obj:`str` or :obj:`int`
    :param framenumbers_str: name of framenumbers datasource
    :type framenumbers_str: :obj:`str`
    :returns:  number of frames
    :rtype: :obj:`str` or :obj:`int`
    """
    return common.blockitem_addint(
        commonblock, framenumbers_str, framenumbers)


def triggermode_cb(commonblock, name, triggermode, saveallimages,
                   framesperfile, height, width, opmode,
                   filepostfix, savefilename_str, framenumbers_str,
                   filename_str, entry_str,
                   shortdetpath=None):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: component name
    :type name: :obj:`str`
    :param triggermode:  trigger mode
    :type triggermode: :obj:`int` or :obj:`str`
    :param saveallimages: save all images flag
    :type saveallimages: :obj:`int` or :obj:`bool`
    :param height: height of the image
    :type height: :obj:`int`
    :param framesperfile: a number of frames per fiel
    :type framesperfile: :obj:`int`
    :param height: height of the image
    :type height: :obj:`int`
    :param width: width of the image
    :type width: :obj:`int`
    :param opmode: operation mode,
                   i.e. 1="int8", 6="int8", 12="int16", 24="int32"
    :type opmode:  :obj:`int`
    :param filepostfix: filename postfix
    :type filepostfix:  :obj:`str`
    :param savefilename_str: name of savefilename datasource
    :type savefilename_str: :obj:`str`
    :param framenumbers_str: name of framenumbers datasource
    :type framenumbers_str: :obj:`str`
    :param filename_str: file name
    :type filename_str: :obj:`str`
    :param entry_str: entry name
    :type entry_str: :obj:`str`
    :param shortdetpath: shortdetpath
    :type shortdetpath: :obj:`bool`
    :returns:  triggermode
    :rtype: :obj:`str` or :obj:`int`
    """

    if saveallimages:

        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
        filenames = []
        framesnumbers = []
        if savefilename_str in commonblock:
            filenames = commonblock[savefilename_str]
        if framenumbers_str in commonblock:
            framesnumbers = commonblock[framenumbers_str]
        fln = min(len(framesnumbers), len(filenames))

        filesframes = []
        lastfile = None
        totalframenumbers = 0
        for fi in range(fln):
            if lastfile != filenames[fi]:
                filesframes.append((filenames[fi], framesnumbers[fi]))
                lastfile = filenames[fi]
                totalframenumbers += framesnumbers[fi]
        dtm = {1: "int8", 6: "int8", 12: "int16", 24: "int32"}
        try:
            dtype = dtm[opmode]
        except Exception:
            dtype = "int32"

        path = ""
        if filename_str:
            sfname = (filename_str).split("/")
            path = sfname[-1].split(".")[0] + "/"
            if shortdetpath is None and \
                    len(sfname) > 1 and sfname[-2] == path[:-1]:
                path = ""
            elif shortdetpath:
                path = ""

        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
            if root.h5object.__class__.__name__ == "File":
                import nxstools.h5pywriter as nxw
            else:
                import nxstools.h5cppwriter as nxw
        else:
            raise Exception("Writer cannot be found")

        en = root.open(entry_str)
        ins = en.open("instrument")
        det = ins.open(name)
        npath = "/entry/instrument/detector/data"

        vfl = nxw.virtual_field_layout(
            [totalframenumbers, height, width], dtype)

        foffset = 0
        for savefilename, framenumbers in filesframes:
            if framenumbers > 0 and framesperfile > 10:
                nbfiles = (framenumbers - 1) // framesperfile + 1
                lastfilenbframes = framenumbers - (nbfiles - 1) * framesperfile
            elif framenumbers > 0:
                nbfiles = 1
                lastfilenbframes = framenumbers
            else:
                nbfiles = 0
                lastfilenbframes = 0

            if nbfiles > 0:
                for nbf in range(0, nbfiles):
                    if framenumbers > framesperfile and framesperfile > 10:
                        connector = "_part%05d." % nbf
                    else:
                        connector = "."
                    filename = path + name + "/" + str(savefilename) \
                        + connector + str(filepostfix)
                    ln = framesperfile if nbf + 1 != nbfiles \
                        else lastfilenbframes
                    ef = nxw.target_field_view(
                        filename, npath, [ln, height, width], dtype)
                    vfl[
                        (foffset + nbf * framesperfile):
                        (foffset + nbf * framesperfile + ln), :, :] = ef
                foffset += framenumbers
        det.create_virtual_field("data", vfl)
    return triggermode



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for lambdavds """

import json
try:
    import tango
except Exception:
    import PyTango as tango


def triggermode_cb(commonblock, name, triggermode,
                   nbimages, hostname, device,
                   filename, stepindex_str, entryname, insname,
                   eigerdectris_str="EigerDectris",
                   eigerfilewriter_str="EigerFileWriter",
                   addfilepattern=False, shape=None,
                   dtype="uint32", acq_modes="",
                   shortdetpath=None):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: component name
    :type name: :obj:`str`
    :param triggermode:  trigger mode
    :type triggermode: :obj:`int` or :obj:`str`
    :param nbimages: a number of images
    :type nbimages: :obj:`int`
    :param hostname: tango host name
    :type hostname: :obj:`str`
    :param device: tango device name
    :type device: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param stepindex_str: name of stepindex datasource
    :type stepindex_str: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param insname: instrument name
    :type insname: :obj:`str`
    :param eigerdectris_str: eigerdectris string
    :type eigerdectris_str: :obj:`str`
    :param eigerfilewriter_str: eigerwriter string
    :type eigerfilewriter_str: :obj:`str`
    :param addfilepattern: add eiger filepattern to the link names
    :type addfilepattern: :obj:`bool`
    :param acq_modes: acquisition modes
    :type acq_modes: :obj:`str`
    :param shortdetpath: shortdetpath
    :type shortdetpath: :obj:`bool`
    :returns: triggermode
    :rtype: :obj:`str` or :obj:`int`
    """

    amodes = acq_modes.split(",")
    host, port = hostname.split(":")
    port = int(port or 10000)
    edb = tango.Database(host, port)

    sl = edb.get_server_list("%s/*" % (eigerdectris_str))
    writer = None
    for ms in sl:
        devserv = edb.get_device_class_list(ms).value_string
        if device in devserv:
            dev = devserv[0::2]
            serv = devserv[1::2]
            for idx, ser in enumerate(serv):
                if ser == eigerfilewriter_str:
                    writer = dev[idx]
                    break
    wp = tango.DeviceProxy('%s/%s' % (hostname, writer))
    filepattern = wp.FilenamePattern.split("/")[-1]
    imagesperfile = wp.ImagesPerFile

    path = ""
    sfname = []
    if filename:
        sfname = (filename).split("/")
        path = sfname[-1].split(".")[0] + "/"
    elif shortdetpath is False and addfilepattern:
        path = filepattern + "/"
    if filename and path:
        if shortdetpath is None and \
                len(sfname) > 1 and sfname[-2] == path[:-1]:
            path = ""
        elif shortdetpath:
            path = ""

    path += '%s/%s_' % (name, filepattern)
    totnbimages = sum(commonblock[stepindex_str])
    nbfiles = (totnbimages + imagesperfile - 1) // imagesperfile
    result = triggermode.lower()
    spf = 0
    cfid = 0
    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if hasattr(root, "currentfileid") and hasattr(root, "stepsperfile"):
            spf = root.stepsperfile
            cfid = root.currentfileid
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise Exception("Writer cannot be found")
    en = root.open(entryname)
    dt = en.open("data")
    ins = en.open(insname)
    det = ins.open(name)
    col = det.open("collection")
    for nbf in range(1, nbfiles+1):
        if addfilepattern:
            fnbf = "%s_data_%06i" % (filepattern, nbf)
            nnbf = fnbf
        else:
            fnbf = "data_%06i" % (nbf)
            nnbf = "%06i" % (nbf)
        if spf > 0 and cfid > 0:
            if cfid == nbf:
                nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                         det, "data")
                nxw.link("/%s/%s/%s/data" % (entryname, insname, name),
                         dt, name)
            nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                     col, "%s" % (fnbf))
        else:
            nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                     col, "%s" % (fnbf))
            nxw.link("/%s/%s/%s/collection/%s" %
                     (entryname, insname, name, fnbf), dt,
                     "%s_%s" % (name, nnbf))
        if "image_filenames" not in col.names():
            tfn = col.create_field("image_filenames", "string", [1])
        else:
            tfn = col.open("image_filenames")
            tfn.grow()
        tfn[int(tfn.shape[0] - 1)] = "%sdata_%06i.h5" % (path, nbf)

    # create VDS field
    if shape and not isinstance(shape, list):
        try:
            shape = json.loads(shape)
        except Exception:
            shape = []
    if not shape or len(shape) < 2:
        return result

    if "nb_images_in_file" not in col.names():
        tni = col.create_field("nb_images_in_file", "uint64")
    else:
        tni = col.open("nb_images_in_file")
        tni.grow()
    tni[int(tni.shape[0] - 1)] = totnbimages
    ttni = tni.read()
    totalframenumbers = int(sum(ttni))
    ttfn = tfn.read()

    if "VDS" in amodes and "data" not in det.names():
        edp = tango.DeviceProxy('%s/%s' % (hostname, device))
        if hasattr(edp, "RoiMode") and edp.RoiMode:
            if hasattr(edp, "RoiModeString") and \
                    hasattr(edp, "RoiYSize") and \
                    edp.RoiModeString == "lines":
                if edp.RoiYSize:
                    try:
                        shape = [int(edp.RoiYSize), shape[1]]
                    except Exception:
                        pass
            else:
                shape = [2167, 2070]

        npath = "/entry/data/data"

        nbimg = []
        nfi = []
        fnms = []
        for ii, tt in enumerate(ttni):
            nbf = int((tt + imagesperfile - 1) // imagesperfile)
            nfi.append(nbf)
            nn = [int(imagesperfile)] * nbf

            fn = [str(ttfn[j + ii * (int(imagesperfile) - 1)])
                  for j in range(nbf)]
            if nn:
                nn[-1] = int(tt - (nbf - 1) * imagesperfile)
            nbimg.extend(nn)
            fnms.extend(fn)

        nboff = [int(sum(nbimg[:(ii)])) for ii in range(len(nbimg))]
        # eiger9m 3110 pixel x 3269 pixel
        # /entry/data/data uint32 [1, 3269 , 3110]
        # eiger4M 2070 pixel x 2167 pixel
        # /entry/data/data uint32 [1, 2167 , 2070]
        # eiger1M  1030 pixel x 1065 pixel
        # /entry/data/data uint32 [1, 1065 , 1030]

        vfl = nxw.virtual_field_layout(
            [totalframenumbers, shape[0], shape[1]], dtype)
        for ii, nb in enumerate(nbimg):
            fnm = fnms[ii]
            ef = nxw.target_field_view(
                fnm, npath, [nb, shape[0], shape[1]], dtype)
            off = nboff[ii]
            vfl.add(
                (slice(off, off + nb), slice(0, shape[0]), slice(0, shape[1])),
                ef, (slice(None), slice(None), slice(None)))

        if "data" not in det.names():
            det.create_virtual_field("data", vfl)
        if name not in dt.names():
            nxw.link("/%s/%s/%s/data" % (entryname, insname, name),
                     dt, name)

    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii
import shutil

try:
    import tango
except Exception:
    import PyTango as tango

# import nxstools
from nxstools import nxscreate

# try:
#     import nxsextrasp00
# except ImportError:
#     from . import nxsextrasp00


try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

try:
    from .checks import checkxmls
except Exception:
    from checks import checkxmls

# try:
#     import TestServerSetUp
# except ImportError:
#     from . import TestServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int


# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreateSECoPCPFSTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            import time
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        self.__args = '{"db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        # home = expanduser("~")
        db = tango.Database()
        self.host = db.get_db_host().split(".")[0]
        self.port = db.get_db_port()
        self.directory = "."
        self.flags = "-d . "
        # self.flags = " -d -r testp09/testmcs/testr228 "
        self.device = 'testp09/testmcs/testr228'
        self.maxDiff = None

        with open("test/files/myuni.xml", "r") as fl:
            self.myuni = fl.read()
        with open("test/files/myuni2.xml", "r") as fl:
            self.myuni2 = fl.read()
        with open("test/files/myunidy.xml", "r") as fl:
            self.myunidy = fl.read()
        with open("test/files/myunidy2.xml", "r") as fl:
            self.myunidy2 = fl.read()

        self.myunids = [
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="CLIENT" name="client_start_time">'
            '\n'
            '    <record name="start_time"/>\n'
            '    <doc>The start time is provided by the control '
            'client.</doc>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_drv">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd("read drv:value", '
            '"", "5000", "0.001", "{cpname}_drv", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__interval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_interval", "", "5000", "0.001", '
            '"{cpname}_drv__interval", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__interval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read drv:_interval", '
            '"", "5000", "0.001", "{cpname}_drv__interval", '
            '[1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__maxcurrent">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_maxcurrent", "", "5000", "0.001", '
            '"{cpname}_drv__maxcurrent", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__maxcurrent_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:_maxcurrent", "", "5000", "0.001", '
            '"{cpname}_drv__maxcurrent", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__move_limit">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_move_limit", "", "5000", "0.001", '
            '"{cpname}_drv__move_limit", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__move_limit_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:_move_limit", "", "5000", "0.001", '
            '"{cpname}_drv__move_limit", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__safe_current">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_safe_current", "", "5000", "0.001", '
            '"{cpname}_drv__safe_current", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__safe_current_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:_safe_current", "", "5000", "0.001", '
            '"{cpname}_drv__safe_current", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_drv__speed">'
            '\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_speed", "", "5000", "0.001", '
            '"{cpname}_drv__speed", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__speed_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:_speed", "", "5000", "0.001", '
            '"{cpname}_drv__speed", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__tolerance">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:_tolerance", "", "5000", "0.001", '
            '"{cpname}_drv__tolerance", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv__tolerance_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:_tolerance", "", "5000", "0.001", '
            '"{cpname}_drv__tolerance", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv_pollinterval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:pollinterval", "", "5000", "0.001", '
            '"{cpname}_drv_pollinterval", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv_pollinterval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:pollinterval", "", "5000", "0.001", '
            '"{cpname}_drv_pollinterval", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_drv_status">'
            '\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd("read drv:status",'
            ' "", "5000", "0.001", "{cpname}_drv_status", [0,0], '
            'commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv_status_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:status", "", "5000", "0.001", '
            '"{cpname}_drv_status", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_drv_target">'
            '\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read drv:target", "", "5000", "0.001", '
            '"{cpname}_drv_target", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_drv_target_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:target", "", "5000", "0.001", '
            '"{cpname}_drv_target", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_drv_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read drv:value", "", "5000", "0.001", '
            '"{cpname}_drv", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_force">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:value", "", "5000", "0.001", '
            '"{cpname}_force", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__adjusting">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_adjusting", "", "5000", "0.001", '
            '"{cpname}_force__adjusting", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__adjusting_current">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_adjusting_current", "", "5000", '
            '"0.001", "{cpname}_force__adjusting_current", [0], '
            'commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__adjusting_current_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_adjusting_current", "", "5000", '
            '"0.001", "{cpname}_force__adjusting_current", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__adjusting_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_adjusting", "", "5000", "0.001", '
            '"{cpname}_force__adjusting", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__current_step">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_current_step", "", "5000", "0.001", '
            '"{cpname}_force__current_step", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__current_step_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_current_step", "", "5000", "0.001", '
            '"{cpname}_force__current_step", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__filter_interval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_filter_interval", "", "5000", "0.001",'
            ' "{cpname}_force__filter_interval", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__filter_interval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_filter_interval", "", "5000", "0.001",'
            ' "{cpname}_force__filter_interval", [1, "t"],'
            ' commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__force_offset">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_force_offset", "", "5000", "0.001", '
            '"{cpname}_force__force_offset", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__force_offset_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_force_offset", "", "5000", "0.001", '
            '"{cpname}_force__force_offset", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__high_pos">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_high_pos", "", "5000", "0.001", '
            '"{cpname}_force__high_pos", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__high_pos_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_high_pos", "", "5000", "0.001", '
            '"{cpname}_force__high_pos", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__hysteresis">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_hysteresis", "", "5000", "0.001", '
            '"{cpname}_force__hysteresis", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__hysteresis_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_hysteresis", "", "5000", "0.001", '
            '"{cpname}_force__hysteresis", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__limit">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_limit", "", "5000", "0.001", '
            '"{cpname}_force__limit", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__limit_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_limit", "", "5000", "0.001", '
            '"{cpname}_force__limit", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__low_pos">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_low_pos", "", "5000", "0.001", '
            '"{cpname}_force__low_pos", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__low_pos_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_low_pos", "", "5000", "0.001", '
            '"{cpname}_force__low_pos", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__pid_i">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_pid_i", "", "5000", "0.001", '
            '"{cpname}_force__pid_i", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__pid_i_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_pid_i", "", "5000", "0.001", '
            '"{cpname}_force__pid_i", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__safe_current">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_safe_current", "", "5000", "0.001", '
            '"{cpname}_force__safe_current", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__safe_current_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_safe_current", "", "5000", "0.001", '
            '"{cpname}_force__safe_current", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__safe_step">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_safe_step", "", "5000", "0.001", '
            '"{cpname}_force__safe_step", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__safe_step_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_safe_step", "", "5000", "0.001", '
            '"{cpname}_force__safe_step", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__slope">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_slope", "", "5000", "0.001", '
            '"{cpname}_force__slope", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__slope_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_slope", "", "5000", "0.001", '
            '"{cpname}_force__slope", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__tolerance">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:_tolerance", "", "5000", "0.001", '
            '"{cpname}_force__tolerance", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force__tolerance_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:_tolerance", "", "5000", "0.001", '
            '"{cpname}_force__tolerance", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_pollinterval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:pollinterval", "", "5000", "0.001", '
            '"{cpname}_force_pollinterval", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_pollinterval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:pollinterval", "", "5000", "0.001", '
            '"{cpname}_force_pollinterval", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_status">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:status", "", "5000", "0.001", '
            '"{cpname}_force_status", [0,0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_status_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read force:status", "", "5000", "0.001", '
            '"{cpname}_force_status", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_target">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read force:target", "", "5000", "0.001", '
            '"{cpname}_force_target", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_target_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read force:target", '
            '"", "5000", "0.001", "{cpname}_force_target", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_force_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read force:value", '
            '"", "5000", "0.001", "{cpname}_force", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_res">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read res:value", "", "5000", "0.001", '
            '"{cpname}_res", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res__jitter">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read res:_jitter", "", "5000", "0.001", '
            '"{cpname}_res__jitter", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res__jitter_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read res:_jitter", '
            '"", "5000", "0.001", "{cpname}_res__jitter", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res_pollinterval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read res:pollinterval", "", "5000", "0.001", '
            '"{cpname}_res_pollinterval", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res_pollinterval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read res:pollinterval", "", "5000", "0.001", '
            '"{cpname}_res_pollinterval", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res_status">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read res:status", "", "5000", "0.001", '
            '"{cpname}_res_status", [0,0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_res_status_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read res:status", '
            '"", "5000", "0.001", "{cpname}_res_status", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_res_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read res:value", "", "5000", "0.001", '
            '"{cpname}_res", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_t">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read T:value", "", "5000", "0.001", "{cpname}_t", '
            '[0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_t__abs">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd("read T:_abs", '
            '"", "5000", "0.001", "{cpname}_t__abs", [0], '
            'commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_t__abs_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd("read T:_abs", "", '
            '"5000", "0.001", "{cpname}_t__abs", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_t__calib">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read T:_calib", "", "5000", "0.001", '
            '"{cpname}_t__calib", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_t__calib_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read T:_calib", "", "5000", "0.001", '
            '"{cpname}_t__calib", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_t_status">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read T:status", "", "5000", "0.001", '
            '"{cpname}_t_status", [0,0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_t_status_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read T:status", "", "5000", "0.001", '
            '"{cpname}_t_status", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" name="{cpname}_t_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read T:value", "", "5000", "0.001", "{cpname}_t", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:value", "", "5000", "0.001", '
            '"{cpname}_transducer", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__friction">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:_friction", "", "5000", "0.001", '
            '"{cpname}_transducer__friction", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__friction_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:_friction", "", "5000", "0.001", '
            '"{cpname}_transducer__friction", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__hysteresis">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:_hysteresis", "", "5000", '
            '"0.001", "{cpname}_transducer__hysteresis", [0], '
            'commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__hysteresis_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:_hysteresis", "", "5000", '
            '"0.001", "{cpname}_transducer__hysteresis", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__jitter">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:_jitter", "", "5000", "0.001", '
            '"{cpname}_transducer__jitter", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__jitter_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:_jitter", "", "5000", "0.001", '
            '"{cpname}_transducer__jitter", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__offset">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:_offset", "", "5000", "0.001", '
            '"{cpname}_transducer__offset", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__offset_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:_offset", "", "5000", "0.001", '
            '"{cpname}_transducer__offset", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__slope">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:_slope", "", "5000", "0.001", '
            '"{cpname}_transducer__slope", [0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer__slope_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:_slope", "", "5000", "0.001", '
            '"{cpname}_transducer__slope", [1, "t"], '
            'commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer_pollinterval">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:pollinterval", "", "5000", '
            '"0.001", "{cpname}_transducer_pollinterval", [0], '
            'commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer_pollinterval_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:pollinterval", "", "5000", '
            '"0.001", "{cpname}_transducer_pollinterval", '
            '[1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer_status">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'ds.result = secop.secop_group_cmd('
            '"read transducer:status", "", "5000", "0.001", '
            '"{cpname}_transducer_status", [0,0], commonblock)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer_status_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:status", "", "5000", "0.001", '
            '"{cpname}_transducer_status", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock, ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n',
            '<?xml version=\'1.0\'?>\n'
            '<definition>\n'
            '  <datasource type="PYEVAL" '
            'name="{cpname}_transducer_time">\n'
            '    <result name="result">\n'
            'from nxstools.pyeval import secop\n'
            'from nxstools.pyeval import timestamp\n'
            'ctime = secop.secop_group_cmd('
            '"read transducer:value", "", "5000", "0.001", '
            '"{cpname}_transducer", [1, "t"], commonblock)\n'
            'ds.result = timestamp.relative_timestamp('
            'commonblock,'
            ' ctime)\n'
            '    </result>\n'
            '  </datasource>\n'
            '</definition>\n'
        ]

        self.secoplist = [
            # 'client_start_time',
            '{cpname}',
            '{cpname}_drv',
            '{cpname}_drv__interval',
            '{cpname}_drv__interval_time',
            '{cpname}_drv__maxcurrent',
            '{cpname}_drv__maxcurrent_time',
            '{cpname}_drv__move_limit',
            '{cpname}_drv__move_limit_time',
            '{cpname}_drv__safe_current',
            '{cpname}_drv__safe_current_time',
            '{cpname}_drv__speed',
            '{cpname}_drv__speed_time',
            '{cpname}_drv__tolerance',
            '{cpname}_drv__tolerance_time',
            '{cpname}_drv_pollinterval',
            '{cpname}_drv_pollinterval_time',
            '{cpname}_drv_status',
            '{cpname}_drv_status_time',
            '{cpname}_drv_target',
            '{cpname}_drv_target_time',
            '{cpname}_drv_time',
            '{cpname}_force',
            '{cpname}_force__adjusting',
            '{cpname}_force__adjusting_current',
            '{cpname}_force__adjusting_current_time',
            '{cpname}_force__adjusting_time',
            '{cpname}_force__current_step',
            '{cpname}_force__current_step_time',
            '{cpname}_force__filter_interval',
            '{cpname}_force__filter_interval_time',
            '{cpname}_force__force_offset',
            '{cpname}_force__force_offset_time',
            '{cpname}_force__high_pos',
            '{cpname}_force__high_pos_time',
            '{cpname}_force__hysteresis',
            '{cpname}_force__hysteresis_time',
            '{cpname}_force__limit',
            '{cpname}_force__limit_time',
            '{cpname}_force__low_pos',
            '{cpname}_force__low_pos_time',
            '{cpname}_force__pid_i',
            '{cpname}_force__pid_i_time',
            '{cpname}_force__safe_current',
            '{cpname}_force__safe_current_time',
            '{cpname}_force__safe_step',
            '{cpname}_force__safe_step_time',
            '{cpname}_force__slope',
            '{cpname}_force__slope_time',
            '{cpname}_force__tolerance',
            '{cpname}_force__tolerance_time',
            '{cpname}_force_pollinterval',
            '{cpname}_force_pollinterval_time',
            '{cpname}_force_status',
            '{cpname}_force_status_time',
            '{cpname}_force_target',
            '{cpname}_force_target_time',
            '{cpname}_force_time',
            '{cpname}_res',
            '{cpname}_res__jitter',
            '{cpname}_res__jitter_time',
            '{cpname}_res_pollinterval',
            '{cpname}_res_pollinterval_time',
            '{cpname}_res_status',
            '{cpname}_res_status_time',
            '{cpname}_res_time',
            '{cpname}_t',
            '{cpname}_t__abs',
            '{cpname}_t__abs_time',
            '{cpname}_t__calib',
            '{cpname}_t__calib_time',
            '{cpname}_t_status',
            '{cpname}_t_status_time',
            '{cpname}_t_time',
            '{cpname}_transducer',
            '{cpname}_transducer__friction',
            '{cpname}_transducer__friction_time',
            '{cpname}_transducer__hysteresis',
            '{cpname}_transducer__hysteresis_time',
            '{cpname}_transducer__jitter',
            '{cpname}_transducer__jitter_time',
            '{cpname}_transducer__offset',
            '{cpname}_transducer__offset_time',
            '{cpname}_transducer__slope',
            '{cpname}_transducer__slope_time',
            '{cpname}_transducer_pollinterval',
            '{cpname}_transducer_pollinterval_time',
            '{cpname}_transducer_status',
            '{cpname}_transducer_status_time',
            '{cpname}_transducer_time'
        ]

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        print("SEED = %s" % self.seed)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        print("tearing down ...")

    def dsexists(self, name):
        return os.path.isfile("%s/%s.ds.xml" % (self.directory, name))

    def cpexists(self, name):
        return os.path.isfile("%s/%s.xml" % (self.directory, name))

    def getds(self, name):
        with open("%s/%s.ds.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def getcp(self, name):
        with open("%s/%s.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def deleteds(self, name):
        os.remove("%s/%s.ds.xml" % (self.directory, name))

    def deletecp(self, name):
        os.remove("%s/%s.xml" % (self.directory, name))

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        nxscreate.main()
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
            etxt = None
        except exception as e:
            error = True
            etxt = str(e)
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er, etxt

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    def checkxmls(self, args, fname):
        """ check xmls of components and datasources
        """

        dstotest = []
        cptotest = []
        try:
            for arg in args:
                # print(arg[0], arg[1])
                skip = False
                for cp in arg[1][0]:
                    if self.cpexists(cp):
                        skip = True
                for ds in arg[1][1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1][1]:
                        dstotest.append(ds)
                    for cp in arg[1][0]:
                        cptotest.append(cp)

                    for cmd in arg[0]:
                        vl, er = self.runtest(cmd)
                        # print(vl)
                        # print(er)
                        if er:
                            self.assertTrue(er.startswith("Info: "))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        for i, ds in enumerate(arg[1][1]):
                            xml = self.getds(ds)
                            checkxmls(
                                self,
                                arg[2][1][i], xml)
                        for i, cp in enumerate(arg[1][0]):
                            xml = self.getcp(cp)
                            checkxmls(
                                self,
                                arg[2][0][i], xml)

                        for ds in arg[1][1]:
                            self.deleteds(ds)
                        for cp in arg[1][0]:
                            self.deletecp(cp)

        finally:
            os.remove(fname)
            for cp in cptotest:
                if self.cpexists(cp):
                    self.deletecp(cp)
            for ds in dstotest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_secopcp_list_none(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """{}"""
        args = [
            [
                ('nxscreate secopcp -l -j %s %s'
                 % (fname, self.flags)).split(),
                ('nxscreate secopcp --list --json-file %s %s'
                 % (fname, self.flags)).split(),
            ],
        ]

        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                # if er:
                #     self.assertTrue(er.startswith("Info: ")
                # else:
                #     self.assertEqual('', er)
                self.assertTrue(not vl)
        finally:
            os.remove(fname)

    def test_secopcp_list_mouldes(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        args = [
            [
                ('nxscreate secopcp -l -j %s %s'
                 % (fname, self.flags)).split(),
                ('nxscreate secopcp --list --json-file %s %s'
                 % (fname, self.flags)).split(),
            ],
        ]

        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        shutil.copy("test/files/secop.conf", fname)
        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                # if er:
                #     self.assertTrue(er.startswith("Info: "))
                # else:
                #     self.assertEqual('', er)
                lines = vl.split("\n")
                self.assertEqual(len(lines), 3)
                self.assertEqual(lines[-3], "MODULES:")
                self.assertEqual(
                    sorted(lines[-2].split()),
                    sorted(['force', 'drv', 'transducer', 'res', 'T']))
                self.assertEqual(
                    lines[-1].split(),
                    [])
        finally:
            os.remove(fname)

    def test_secopcp_list_mouldes_dict(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        args = [
            [
                ('nxscreate secopcp -l -j %s %s'
                 % (fname, self.flags)).split(),
                ('nxscreate secopcp --list --json-file %s %s'
                 % (fname, self.flags)).split(),
            ],
        ]

        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        shutil.copy("test/files/secop2.conf", fname)
        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                # if er:
                #     self.assertTrue(er.startswith("Info: "))
                # else:
                #     self.assertEqual('', er)
                lines = vl.split("\n")
                self.assertEqual(len(lines), 3)
                self.assertEqual(lines[-3], "MODULES:")
                self.assertEqual(
                    sorted(lines[-2].split()),
                    sorted(['force', 'drv', 'transducer', 'res', 'T']))
                self.assertEqual(
                    lines[-1].split(),
                    [])
        finally:
            os.remove(fname)

    def ttest_secopcp_create_old(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "myuni"
        secoplist = [cc.format(cpname=cname) for cc in self.secoplist]

        args = [
            [
                ('nxscreate secopcp -c %s -j %s %s'
                 % (cname, fname, self.flags)).split(),
                ('nxscreate secopcp --component %s --json-file %s %s'
                 % (cname, fname, self.flags)).split(),
            ],
        ]

        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        shutil.copy("test/files/secop.conf", fname)
        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                # if er:
                #     self.assertTrue(er.startswith("Info: "))
                # else:
                #     self.assertEqual('', er)
                lines = vl.split("\n")
                # print(lines[:10])
                self.assertEqual(len(lines), 135)
                self.assertEqual(lines[0], "OUTPUT DIRECTORY: .")
                self.assertEqual(len(lines), 135)
                ncst = sorted(
                    [ll for ll in lines[1:]
                     if ('client_start_time' not in ll and ll)])
                cst = sorted(
                    [ll for ll in lines[1:]
                     if 'client_start_time' in ll])
                self.assertEqual(len(cst), 44)
                self.assertEqual(len(set(cst)), 1)
                fmtstr = "CREATING '{name}' of secop in './{name}.ds.xml'"
                cfmtstr = "CREATING '{name}' of secop in './{name}.xml'"
                self.assertEqual(
                    list(set(cst))[0],
                    fmtstr.format(name='client_start_time'))
                self.assertEqual(len(ncst), 89)
                self.assertEqual(len(ncst), len(secoplist))
                for si, scn in enumerate(secoplist):
                    # print(scn, ncst[si])
                    if "_" in scn:
                        self.assertEqual(ncst[si], fmtstr.format(name=scn))
                    else:
                        self.assertEqual(ncst[si], cfmtstr.format(name=scn))

        finally:
            os.remove(fname)
            for scn in secoplist:
                pass
                # if "_" in scn:
                #     os.remove("%s.ds.xml" % scn)
                # else:
                #     os.remove("%s.xml" % scn)

    def test_secopcp_create(self):
        """ test nxsccreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "myuni"

        shutil.copy("test/files/secop.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp -c %s -j %s %s'
                     % (cname, fname, self.flags)).split(),
                    ('nxscreate secopcp --component %s --json-file %s %s'
                     % (cname, fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myuni.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)

    def test_secopcp_create_dict(self):
        """ test nxsccreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "myuni"

        shutil.copy("test/files/secop2.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp -c %s -j %s %s'
                     % (cname, fname, self.flags)).split(),
                    ('nxscreate secopcp --component %s --json-file %s %s'
                     % (cname, fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myuni2.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)

    def test_secopcp_create_dynamic_nxdata(self):
        """ test nxsccreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "myunids2"

        shutil.copy("test/files/secop.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp -c %s -j %s %s --dynamic'
                     ' --sample-nxdata '
                     % (cname, fname, self.flags)).split(),
                    ('nxscreate secopcp --dynamic --component %s '
                     '--sample-nxdata --json-file %s %s'
                     % (cname, fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myunidy.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)

    def test_secopcp_create_dynamic_nxdata_dict(self):
        """ test nxsccreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "myunids2"

        shutil.copy("test/files/secop2.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp -c %s -j %s %s --dynamic'
                     ' --sample-nxdata '
                     % (cname, fname, self.flags)).split(),
                    ('nxscreate secopcp --dynamic --component %s '
                     '--sample-nxdata --json-file %s %s'
                     % (cname, fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myunidy2.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)

    def test_secopcp_create_def(self):
        """ test nxscreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "uniax_sim"

        shutil.copy("test/files/secop.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp  -j %s %s'
                     % (fname, self.flags)).split(),
                    ('nxscreate secopcp  --json-file %s %s'
                     % (fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myuni.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)

    def test_secopcp_create_def_dict(self):
        """ test nxscreate stdcomp file system
        """

        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.json' % (
            os.getcwd(), self.__class__.__name__, fun)

        cname = "uniax_sim"

        shutil.copy("test/files/secop2.conf", fname)

        dsl = ["client_start_time"]
        dsl.extend(self.secoplist[1:])
        args = [
            [
                [
                    ('nxscreate secopcp  -j %s %s'
                     % (fname, self.flags)).split(),
                    ('nxscreate secopcp  --json-file %s %s'
                     % (fname, self.flags)).split(),
                ],
                [
                    [self.secoplist[0].format(cpname=cname)],
                    [ds.format(cpname=cname) for ds in dsl]
                ],
                [
                    [
                        self.myuni2.format(cpname=cname)
                    ],
                    [ds.format(cpname=cname) for ds in self.myunids]
                ],
            ],
        ]
        self.checkxmls(args, fname)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for perkinelmerdetector """


def fileindex_cb(commonblock, name, value):
    """ add block item to commonblock

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: name of block item
    :type name: :obj:`str`
    :param value: item value
    :type value: :obj:`str`
    :returns: item value
    :rtype: :obj:`str`
    """
    if name not in commonblock:
        commonblock[name] = value
    return value - 1


def postrun(commonblock,
            outputdirectory,
            filepattern,
            filename,
            fileindex,
            fileindex_str):
    """ code for postrun  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param outputdirectory: file directorentry name
    :type outputdirectory: :obj:`str`
    :param filepattern: file pattern
    :type filepattern: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param fileindex: file index
    :type fileindex: :obj:`int`
    :param fileindex_str: file index string
    :type fileindex_str: :obj:`str`
    :returns: postrun string
    :rtype: :obj:`str`
    """
    unixdir = str(outputdirectory).replace("\\", "/")
    startfileindex = commonblock[fileindex_str] - 1
    lastfileindex = fileindex - 1
    if len(unixdir) > 1 and unixdir[1] == ":":
        unixdir = "/data" + unixdir[2:]
    if unixdir and unixdir[-1] == "/":
        unixdir = unixdir[:-1]
    result = "" + unixdir + "/" + filepattern + "-%05d."
    result += str(filename.split(".")[-1])
    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if hasattr(root, "currentfileid") and hasattr(root, "stepsperfile"):
            spf = root.stepsperfile
            cfid = root.currentfileid
            if spf > 0 and cfid > 0:
                lastfileindex = min(
                    startfileindex + cfid * spf - 1, lastfileindex)
                startfileindex = startfileindex + (cfid - 1) * spf
    result += ":" + str(startfileindex) + ":" + str(lastfileindex)
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for lambda """


def m2_external_data(commonblock,
                     name,
                     savefilename,
                     saveallimages,
                     filepostfix,
                     filename,
                     modulename,
                     shortdetpath=None):
    """ code for external_data datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: detector name
    :type name: :obj:`str`
    :param savefilename: name of saved file
    :type savefilename: :obj:`str`
    :param saveallimages: save all images flag
    :type saveallimages: :obj:`bool` or :obj:`int`
    :param filepostfix: file postfix
    :type filepostfix: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param filename: module name
    :type filename: :obj:`str`
    :param shortdetpath: shortdetpath
    :type shortdetpath: :obj:`bool`
    :returns: name of saved file
    :rtype: :obj:`str`
    """
    result = ""
    if saveallimages:
        if filename:
            sfname = (filename).split("/")
            result = sfname[-1].split(".")[0] + "/"
            if shortdetpath is None and \
                    len(sfname) > 1 and sfname[-2] == result[:-1]:
                result = ""
            elif shortdetpath:
                result = ""
        result += name + "/" + str(savefilename) + "_" + modulename \
            + "." + str(filepostfix) + "://entry/instrument/detector"
    return result


def external_data(commonblock,
                  name,
                  savefilename,
                  saveallimages,
                  framesperfile,
                  framenumbers,
                  filepostfix,
                  filename,
                  shortdetpath=None):
    """ code for external_data datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: detector name
    :type name: :obj:`str`
    :param savefilename: name of saved file
    :type savefilename: :obj:`str`
    :param saveallimages: save all images flag
    :type saveallimages: :obj:`bool` or :obj:`int`
    :param framesperfile: frames per file
    :type framesperfile: :obj:`int`
    :param framenumbers: frames per file
    :type framenumbers: :obj:`int`
    :param filepostfix: file postfix
    :type filepostfix: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param shortdetpath: shortdetpath
    :type shortdetpath: :obj:`bool`
    :returns: name of saved file
    :rtype: :obj:`str`
    """
    result = ""
    if saveallimages:
        if filename:
            sfname = (filename).split("/")
            result = sfname[-1].split(".")[0] + "/"
            if shortdetpath is None and \
                    len(sfname) > 1 and sfname[-2] == result[:-1]:
                result = ""
            elif shortdetpath:
                result = ""
        fpf = framesperfile
        fn = framenumbers
        spf = 0
        cfid = 0
        if fpf != fn:
            if "__root__" in commonblock.keys():
                root = commonblock["__root__"]
                if hasattr(root, "currentfileid") and \
                   hasattr(root, "stepsperfile"):
                    spf = root.stepsperfile
                    cfid = root.currentfileid
        if spf > 0 and cfid > 0:
            result += name + "/" + str(savefilename) \
                + "_part%05d." % (cfid - 1) + str(filepostfix) \
                + "://entry/instrument/detector"
        else:
            result += name + "/" + str(savefilename) \
                + "." + str(filepostfix) + "://entry/instrument/detector"
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfigurator_test.py
# unittests for field Tags running Tango Server
#
import unittest
import sys
import shutil
import os


try:
    import NXSCreateClientDSFS_test
except Exception:
    from . import NXSCreateClientDSFS_test


if sys.version_info > (3,):
    unicode = str
    long = int


# test fixture
class NXSCreateClientDSFS2Test(
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.__init__(
            self, methodName)

        self.directory = "my_test_nxs"
        self._dircreated = False
        self.flags = " -d %s" % self.directory

    # test starter
    # \brief Common set up
    def setUp(self):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.setUp(self)
        if not os.path.isdir(self.directory):
            os.makedirs(self.directory)
            self._dircreated = True

    # test closer
    # \brief Common tear down
    def tearDown(self):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.tearDown(self)
        if self._dircreated:
            shutil.rmtree(self.directory)
            self._dircreated = False

    def test_clientds_file_prefix(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -x test_ -v exp_vc  -f3 -l5 %s'
                 % self.flags).split(),
                ['test_exp_vc03',
                 'test_exp_vc04',
                 'test_exp_vc05'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="exp_vc03">
    <record name="exp_vc03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="exp_vc04">
    <record name="exp_vc04"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="exp_vc05">
    <record name="exp_vc05"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds --device dd '
                 '--file-prefix test_exp_ '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
                ['test_exp_dd03',
                 'test_exp_dd04'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="dd03">
    <record name="dd03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="dd04">
    <record name="dd04"/>
  </datasource>
</definition>
""",
                ],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            pass
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for dcm """

try:
    import tango
except Exception:
    import PyTango as tango


def unitcalibration(dcmdevice):
    """ code for dcm unitcalibration datasource

    :param dcmdevice:  dcm device name
    :type dcmdevice: :obj:`str`
    :returns: unit calibration
    :rtype: :obj:`float`
    """
    dp = tango.DeviceProxy(dcmdevice)
    bdv = dp.get_property("BraggDevice")['BraggDevice'][0]
    bdp = tango.DeviceProxy(bdv)
    return bdp.unitcalibration


def reflection(dcmdevice):
    """ code for dcm crystal reflection datasource

    :param dcmdevice:  dcm device name
    :type dcmdevice: :obj:`str`
    :returns: dcm crystal reflection
    :rtype: :obj:`list` <:obj:`int`>
    """
    dp = tango.DeviceProxy(dcmdevice)
    version = dp.get_property("Version")['Version'][0]
    crystal = dp.crystal
    if version != '11':
        return [3, 1, 1] if crystal == 1 else [1, 1, 1]
    else:
        return [2, 2, 0] if crystal == 1 else [1, 1, 1]


def crystal(dcmdevice):
    """ code for dcm crystal datasource

    :param dcmdevice:  dcm device name
    :type dcmdevice: :obj:`str`
    :returns: dcm crystal
    :rtype: :obj:`in`
    """
    dp = tango.DeviceProxy(dcmdevice)
    return dp.crystal



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  PaNET ontology dictionary """

import json
import os


def read_techniques():
    """ read PaNET ontology techniques to dictionary

    :returns: techniques id:label
    :rtype: :obj:`dict` <:obj:`str`,:obj:`str`>
    """
    result = {}
    with open(os.path.join(os.path.dirname(__file__), "ontology.json")) as fp:
        ont = json.loads(fp.read())
    ott = [on for on in ont
           if "@type" in on
           and 'http://www.w3.org/2002/07/owl#Class' in on["@type"]
           and "@id" in on and
           on["@id"].startswith("http://purl.org/pan-science/PaNET/PaNET")]
    result = {
        ot["@id"]: ot['http://www.w3.org/2000/01/rdf-schema#label'][0][
            "@value"] for ot in ott}
    return result


#: (:obj:`dict` <:obj:`str`,:obj:`str` >)
#:     techniques id:label
id_techniques = read_techniques()

#: (:obj:`dict` <:obj:`str`,:obj:`str` >)
#:     nexus application  to PaNET
nexus_panet = {
    "saxs": "http://purl.org/pan-science/PaNET/PaNET01188",
    "waxs": "http://purl.org/pan-science/PaNET/PaNET01191",
    #     This is a definition for data to be archived by ICAT
    # (http://www.icatproject.org/).
    # "archive": "",
    #     This is an application definition for angular
    # resolved photo electron spectroscopy.
    # "arpes": "",
    #     Implementation of the canSAS standard to store
    # reduced small-angle scattering data of any dimension.
    # "canSAS": "",
    #     This is a application definition for raw data
    # from a direct geometry TOF spectrometer
    # "directtof": "",
    #     This is an application definition for raw data from an
    # X-ray fluorescence experiment
    # "fluo": "",
    #     This is a application definition for raw data from a
    # direct geometry TOF spectrometer
    # "indirecttof": "",
    #     Application definition for any data.
    # "iqproc": "",
    #     This is the application definition for a TOF laue diffractometer
    # "lauetof": "",
    #     Monochromatic Neutron and X-Ray Powder diffractometer
    # "monopd": "",
    #     functional application definition for macromolecular crystallography
    # "mx": "",
    #     This is an application definition for a monochromatic
    # scanning reflectometer.
    # "refscan": "",
    #     This is an application definition for raw data from
    # a TOF reflectometer.
    # "reftof": "",
    #     raw, monochromatic 2-D SAS data with an area detector
    # "sas": "",
    #     raw, 2-D SAS data with an area detector with a time-of-flight source
    # "sastof": "",
    #     Application definition for a generic scan instrument.
    # "scan": "",
    #     NXSPE Inelastic Format. Application definition for NXSPE file format.
    # "spe": "",
    #     This is the application definition for S(Q,OM) processed data.
    # "sqom": "",
    #     Application definition for a STXM instrument.
    # "stxm": "",
    #     This is an application definition for a triple axis spectrometer.
    # "tas": "",
    #     This is a application definition for raw data from
    # a TOF neutron powder diffractometer
    # "tofnpd": "",
    #     This is an application definition for raw data from
    # a generic TOF instrument
    # "tofraw": "",
    #     This is a application definition for raw data from
    # a generic TOF instrument
    # "tofsingle": "",
    #     This is the application definition for x-ray
    # or neutron tomography raw data.
    # "tomo": "",
    #     This is the application definition for x-ray or neutron tomography
    # raw data with phase contrast variation at each point.
    # "tomophase": "",
    #     This is an application definition for the final result of
    # a tomography experiment: a 3D construction of some volume of physical
    # properties.
    # "tomoproc": "",
    #     This is an application definition for raw data from an X-ray
    # absorption spectroscopy experiment.
    # "xas": "",
    #     Processed data from XAS. This is energy versus
    # I(incoming)/I(absorbed).
    # "xasproc": "",
    #     This definition covers the common parts of all monochromatic
    # single crystal raw data application definitions.
    # "xbase": "",
    #     raw data from a four-circle diffractometer with an eulerian
    # cradle, extends xbase
    # "xeuler": "",
    #     raw data from a kappa geometry (CAD4) single crystal
    # diffractometer, extends NXxbase
    # "xkappa": "",
    #     raw data from a single crystal laue camera, extends NXxrot
    # "xlaue": "",
    #     raw data from a single crystal Laue camera, extends NXxlaue
    # "xlaueplate": "",
    #     raw data from a single crystal diffractometer, extends NXxbase
    # "xnb": "",
    #     raw data from a rotation camera, extends NXxbase
    # "xrot": "",
}
